@@ -842,7 +842,7 @@ double CSSPrimitiveValueImpl::computeLengthFloat( khtml::RenderStyle *style, QPa
         return -1;
     }
 
-    return getFloatValue(type)*factor;
+    return getFloatValue() * factor;
 }
 
 void CSSPrimitiveValueImpl::setFloatValue( unsigned short unitType, double floatValue, int &exceptioncode )
@@ -859,6 +859,54 @@ void CSSPrimitiveValueImpl::setFloatValue( unsigned short unitType, double float
     m_type = unitType;
 }
 
+double scaleFactorForConversion(unsigned short unitType)
+{
+    double cssPixelsPerInch = 96.0;
+    double factor = 1.0;
+    
+    switch(unitType) {
+        case CSSPrimitiveValue::CSS_PX:
+            break;
+        case CSSPrimitiveValue::CSS_CM:
+            factor = cssPixelsPerInch / 2.54; // (2.54 cm/in)
+            break;
+        case CSSPrimitiveValue::CSS_MM:
+            factor = cssPixelsPerInch / 25.4;
+            break;
+        case CSSPrimitiveValue::CSS_IN:
+            factor = cssPixelsPerInch;
+            break;
+        case CSSPrimitiveValue::CSS_PT:
+            factor = cssPixelsPerInch / 72.0;
+            break;
+        case CSSPrimitiveValue::CSS_PC:
+            factor = cssPixelsPerInch * 12.0 / 72.0; // 1 pc == 12 pt
+            break;
+        default:
+            break;
+    }
+    
+    return factor;
+}
+
+double CSSPrimitiveValueImpl::getFloatValue(unsigned short unitType)
+{
+    if (unitType == m_type || unitType < CSSPrimitiveValue::CSS_PX || unitType > CSSPrimitiveValue::CSS_PC)
+        return m_value.num;
+    
+    double convertedValue = m_value.num;
+    
+    // First convert the value from m_type into CSSPixels
+    double factor = scaleFactorForConversion(m_type);
+    convertedValue *= factor;
+    
+    // Now convert from CSSPixels to the specified unitType
+    factor = scaleFactorForConversion(unitType);
+    convertedValue /= factor;
+    
+    return convertedValue;
+}
+
 void CSSPrimitiveValueImpl::setStringValue( unsigned short stringType, const DOMString &stringValue, int &exceptioncode )
 {
     exceptioncode = 0;
