@@ -159,7 +159,7 @@ namespace KJS {
            SetAttributeNode, RemoveAttributeNode, GetElementsByTagName,
            GetAttributeNS, SetAttributeNS, RemoveAttributeNS, GetAttributeNodeNS,
            SetAttributeNodeNS, GetElementsByTagNameNS, HasAttribute, HasAttributeNS,
-           ScrollByLines, ScrollByPages, ScrollIntoView};
+           ScrollByLines, ScrollByPages, ScrollIntoView, ElementFocus, ElementBlur};
   };
 
   class DOMDOMImplementation : public DOMObject {
