@@ -65,7 +65,7 @@ class HTMLOptionsCollectionImpl;
 class HTMLFormElementImpl : public HTMLElementImpl
 {
 public:
-    HTMLFormElementImpl(DocumentPtr *doc);
+    HTMLFormElementImpl(DocumentImpl *doc);
     virtual ~HTMLFormElementImpl();
 
     virtual Id id() const;
@@ -145,7 +145,7 @@ class HTMLGenericFormElementImpl : public HTMLElementImpl
     friend class khtml::RenderFormElement;
 
 public:
-    HTMLGenericFormElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLGenericFormElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
     virtual ~HTMLGenericFormElementImpl();
 
     HTMLFormElementImpl *form() { return m_form; }
@@ -213,7 +213,7 @@ protected:
 class HTMLButtonElementImpl : public HTMLGenericFormElementImpl
 {
 public:
-    HTMLButtonElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLButtonElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
 
     virtual ~HTMLButtonElementImpl();
 
@@ -251,7 +251,7 @@ protected:
 class HTMLFieldSetElementImpl : public HTMLGenericFormElementImpl
 {
 public:
-    HTMLFieldSetElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLFieldSetElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
 
     virtual ~HTMLFieldSetElementImpl();
 
@@ -297,7 +297,7 @@ public:
 #endif
     };
 
-    HTMLInputElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLInputElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
     virtual ~HTMLInputElementImpl();
 
     virtual Id id() const;
@@ -403,7 +403,7 @@ protected:
 class HTMLLabelElementImpl : public HTMLElementImpl
 {
 public:
-    HTMLLabelElementImpl(DocumentPtr *doc);
+    HTMLLabelElementImpl(DocumentImpl *doc);
     virtual ~HTMLLabelElementImpl();
 
     virtual bool isFocusable() const;
@@ -428,7 +428,7 @@ public:
 class HTMLLegendElementImpl : public HTMLGenericFormElementImpl
 {
 public:
-    HTMLLegendElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLLegendElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
     virtual ~HTMLLegendElementImpl();
 
     virtual bool isFocusable() const;
@@ -446,7 +446,7 @@ class HTMLSelectElementImpl : public HTMLGenericFormElementImpl
     friend class khtml::RenderSelect;
 
 public:
-    HTMLSelectElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLSelectElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
     ~HTMLSelectElementImpl();
 
     virtual Id id() const;
@@ -532,7 +532,7 @@ protected:
 class HTMLKeygenElementImpl : public HTMLSelectElementImpl
 {
 public:
-    HTMLKeygenElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLKeygenElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
 
     virtual Id id() const;
 
@@ -553,7 +553,7 @@ protected:
 class HTMLOptGroupElementImpl : public HTMLGenericFormElementImpl
 {
 public:
-    HTMLOptGroupElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLOptGroupElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
     virtual ~HTMLOptGroupElementImpl();
 
     virtual Id id() const;
@@ -580,7 +580,7 @@ class HTMLOptionElementImpl : public HTMLGenericFormElementImpl
     friend class DOM::HTMLSelectElementImpl;
 
 public:
-    HTMLOptionElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLOptionElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
 
     virtual bool isFocusable() const;
     
@@ -621,7 +621,7 @@ public:
         ta_Physical
     };
 
-    HTMLTextAreaElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLTextAreaElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
     ~HTMLTextAreaElementImpl();
 
     virtual Id id() const;
@@ -687,7 +687,7 @@ protected:
 class HTMLIsIndexElementImpl : public HTMLInputElementImpl
 {
 public:
-    HTMLIsIndexElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f = 0);
+    HTMLIsIndexElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f = 0);
 
     virtual Id id() const;
     virtual void parseHTMLAttribute(HTMLAttributeImpl *attr);
