@@ -37,7 +37,7 @@ class HTMLCollection;
 class HTMLBaseFontElementImpl : public HTMLElementImpl
 {
 public:
-    HTMLBaseFontElementImpl(DocumentPtr *doc);
+    HTMLBaseFontElementImpl(DocumentImpl *doc);
 
     ~HTMLBaseFontElementImpl();
 
