@@ -52,6 +52,7 @@
 
 #include <khtml_settings.h>
 #include <dom/dom_string.h>
+#include "DocPtr.h"
 
 class QMovie;
 class KHTMLPart;
@@ -97,7 +98,9 @@ namespace khtml
         LRUList();
         ~LRUList();
     };
-    
+
+    bool isXMLMIMEType(const QString& mimeType);
+
     /**
      * @internal
      *
