@@ -98,7 +98,7 @@ public:
     // iterate over structures on the line.
     virtual InlineBox* inlineBoxWrapper() const;
     virtual void setInlineBoxWrapper(InlineBox* b);
-    void deleteLineBoxWrapper();
+    virtual void deleteLineBoxWrapper();
     
     virtual int lowestPosition(bool includeOverflowInterior=true, bool includeSelf=true) const;
     virtual int rightmostPosition(bool includeOverflowInterior=true, bool includeSelf=true) const;
