@@ -182,6 +182,15 @@ void RenderFlow::deleteLineBoxes()
 
 void RenderFlow::detach()
 {
+    // Detach our continuation first.
+    if (m_continuation)
+        m_continuation->detach();
+    m_continuation = 0;
+
+    // Make sure to destroy anonymous children first while they are still connected to the rest of the tree, so that they will
+    // properly dirty line boxes that they are removed from.  Effects that do :before/:after only on hover could crash otherwise.
+    RenderContainer::detachLeftoverChildren();
+     
     if (!documentBeingDestroyed()) {
         if (m_firstLineBox) {
             // We can't wait for RenderContainer::detach to clear the selection,
