@@ -377,7 +377,7 @@ public:
     // iterate over structures on the line.
     virtual InlineBox* inlineBoxWrapper() const;
     virtual void setInlineBoxWrapper(InlineBox* b);
-    void deleteLineBoxWrapper();
+    virtual void deleteLineBoxWrapper();
 
     virtual InlineBox *inlineBox(long offset=0, EAffinity affinity = UPSTREAM);
     
