@@ -799,7 +799,7 @@ void RenderText::setStyle(RenderStyle *_style)
         RenderObject::setStyle( _style );
 
         if (needToTransformText) {
-            SharedPtr<DOMStringImpl> textToTransform = originalString();
+            khtml::SharedPtr<DOMStringImpl> textToTransform = originalString();
             if (textToTransform.notNull())
                 setText(textToTransform.get(), true);
         }
@@ -894,9 +894,9 @@ bool RenderText::isTextFragment() const
     return false;
 }
 
-SharedPtr<DOMStringImpl> RenderText::originalString() const
+khtml::SharedPtr<DOMStringImpl> RenderText::originalString() const
 {
-    return element() ? SharedPtr<DOMStringImpl>(element()->string()) : SharedPtr<DOMStringImpl>();
+    return element() ? khtml::SharedPtr<DOMStringImpl>(element()->string()) : khtml::SharedPtr<DOMStringImpl>();
 }
 
 void RenderText::absoluteRects(QValueList<QRect>& rects, int _tx, int _ty)
@@ -1884,7 +1884,7 @@ bool RenderTextFragment::isTextFragment() const
     return true;
 }
 
-SharedPtr<DOMStringImpl> RenderTextFragment::originalString() const
+khtml::SharedPtr<DOMStringImpl> RenderTextFragment::originalString() const
 {
     DOM::DOMStringImpl* result = 0;
     if (element())
@@ -1893,7 +1893,7 @@ SharedPtr<DOMStringImpl> RenderTextFragment::originalString() const
         result = contentString();
     if (result && (start() > 0 || start() < result->l))
         result = result->substring(start(), end());
-    return SharedPtr<DOMStringImpl>(result);
+    return khtml::SharedPtr<DOMStringImpl>(result);
 }
 #undef BIDI_DEBUG
 #undef DEBUG_LAYOUT
