@@ -329,18 +329,6 @@ static bool     isStreamer (Position pos)
     return (pos.offset() == 0);
 }
 
-// AFAIK no one has a clear, complete definition for this method and how it is used.
-// Here is what I have come to understand from re-working the code after fixing PositionIterator
-// for <rdar://problem/4103339>.  See also Ken's comments in the header.  Fundamentally, upstream()
-// scans backward in the DOM starting at "this" to return a visible DOM position that is either in
-// a text node, or just after a replaced or BR element (btw downstream() also considers empty blocks).
-// If "stayInBlock" is specified, the search stops when it would have entered into a part of the DOM
-// with a different enclosing block, including a nested one.  Otherwise, the search stops at the start
-// of the entire DOM tree.  If "stayInBlock" stops the search, this method returns the highest previous
-// position that is either in an atomic node (i.e. text) or is the end of a non-atomic node
-// (_regardless_ of visibility).  If the end-of-DOM stopped the search, this method returns the 
-// highest previous visible node that is either in an atomic node (i.e. text) or is the end of a
-// non-atomic node.
 Position Position::upstream(EStayInBlock stayInBlock) const
 {
     // start at equivalent deep position
@@ -354,12 +342,12 @@ Position Position::upstream(EStayInBlock stayInBlock) const
     Position lastVisible = *this;
     Position lastStreamer = *this;
     Position currentPos = start;
-    for (; !currentPos.atStart(); currentPos = currentPos.previous()) {
+    for (; !currentPos.atStart(); currentPos = currentPos.previous(UsingComposedCharacters)) {
         NodeImpl *currentNode = currentPos.node();
         int currentOffset = currentPos.offset();
 
-        // limit traversal to block or table enclosing the original element
-        // NOTE: This includes not going into nested blocks
+        // Don't enter a new enclosing block flow or table element.  There is code below that
+        // terminates early if we're about to leave an enclosing block flow or table element.
         if (stayInBlock && block != currentNode->enclosingBlockFlowOrTableElement())
             return lastStreamer;
 
@@ -375,6 +363,13 @@ Position Position::upstream(EStayInBlock stayInBlock) const
         // track last visible streamer position
         if (isStreamer(currentPos))
             lastVisible = currentPos;
+        
+        // Don't leave a block flow or table element.  We could rely on code above to terminate and 
+        // return lastVisible on the next iteration, but we terminate early to avoid calling previous()
+        // beceause previous() for an offset 0 position calls nodeIndex(), which is O(n).
+        // FIXME: Avoid calling previous on other offset 0 positions.
+        if (stayInBlock && currentNode == currentNode->enclosingBlockFlowOrTableElement() && currentOffset == 0)
+            return lastVisible;
 
         // return position after replaced or BR elements
         if (renderer->isReplaced() || renderer->isBR()) {
@@ -414,17 +409,6 @@ Position Position::upstream(EStayInBlock stayInBlock) const
     return lastVisible;
 }
 
-// AFAIK no one has a clear, complete definition for this method and how it is used.
-// Here is what I have come to understand from re-working the code after fixing PositionIterator
-// for <rdar://problem/4103339>.  See also Ken's comments in the header.  Fundamentally, downstream()
-// scans forward in the DOM starting at "this" to return the first visible DOM position that is
-// either in a text node, or just before a replaced, BR element, or empty block flow element (i.e.
-// non-text nodes with no children).  If "stayInBlock" is specified, the search stops when it would
-// have entered into a part of the DOM with a different enclosing block, including a nested one.
-// Otherwise, the search stops at the end of the entire DOM tree.  If "stayInBlock" stops the search,
-// this method returns the first previous position that is either in an atomic node (i.e. text) or is
-// at offset 0 (_regardless_ of visibility).  If the end-of-DOM stopped the search, this method returns
-// the first previous visible node that is either in an atomic node (i.e. text) or is at offset 0.
 Position Position::downstream(EStayInBlock stayInBlock) const
 {
     // start at equivalent deep position
@@ -438,7 +422,7 @@ Position Position::downstream(EStayInBlock stayInBlock) const
     Position lastVisible = *this;
     Position lastStreamer = *this;
     Position currentPos = start;
-    for (; !currentPos.atEnd(); currentPos = currentPos.next()) {   
+    for (; !currentPos.atEnd(); currentPos = currentPos.next(UsingComposedCharacters)) {   
         NodeImpl *currentNode = currentPos.node();
         int currentOffset = currentPos.offset();
 
@@ -447,9 +431,7 @@ Position Position::downstream(EStayInBlock stayInBlock) const
         if (currentNode->id() == ID_BODY && currentOffset >= (int) currentNode->childNodeCount())
             break;
             
-        // limit traversal to block or table enclosing the original element
-        // return the last streamer position regardless of visibility
-        // NOTE: This includes not going into nested blocks
+        // Do not enter a new enclosing block flow or table element, and don't leave the original one.
         if (stayInBlock && block != currentNode->enclosingBlockFlowOrTableElement())
             return lastStreamer;
         
@@ -466,28 +448,6 @@ Position Position::downstream(EStayInBlock stayInBlock) const
         if (isStreamer(currentPos))
             lastVisible = currentPos;
 
-        // if now at a offset 0 of a rendered block flow element...
-        //      - return current position if the element has no children (i.e. is a leaf)
-        //      - return child node, offset 0, if the first visible child is not a block flow element
-        //      - otherwise, skip this position (first visible child is a block, and we will
-        //          get there eventually via the iterator)
-        if ((currentNode != startNode && renderer->isBlockFlow()) && (currentOffset == 0)) {
-            if (!currentNode->firstChild())
-                return currentPos;
-                
-            for (NodeImpl *child = currentNode->firstChild(); child; child = child->nextSibling()) {
-                RenderObject *r = child->renderer();
-                if (r && r->style()->visibility() == VISIBLE) {
-                    if (r->isBlockFlow())
-                        break; // break causes continue code below to run.
-
-                    return Position(child, 0);
-                }
-            }
-
-            continue;
-        }
-
         // return position before replaced or BR elements
         if (renderer->isReplaced() || renderer->isBR()) {
             if (currentOffset <= renderer->caretMinOffset())
@@ -602,11 +562,10 @@ bool Position::inRenderedContent() const
     if (renderer->style()->visibility() != VISIBLE)
         return false;
 
-    // FIXME: This check returns false for a <br> at the end of a line!
-    if (renderer->isBR() && static_cast<RenderText *>(renderer)->firstTextBox()) {
+    if (renderer->isBR())
         return offset() == 0;
-    }
-    else if (renderer->isText()) {
+    
+       if (renderer->isText()) {
         RenderText *textRenderer = static_cast<RenderText *>(renderer);
         for (InlineTextBox *box = textRenderer->firstTextBox(); box; box = box->nextTextBox()) {
             if (offset() >= box->m_start && offset() <= box->m_start + box->m_len) {
@@ -650,7 +609,8 @@ bool Position::inRenderedText() const
             return false;
         }
         if (offset() >= box->m_start && offset() <= box->m_start + box->m_len)
-            return true;
+            // Return false for offsets inside composed characters.
+            return offset() == 0 || offset() == textRenderer->nextOffset(textRenderer->previousOffset(offset()));
     }
     
     return false;
