@@ -27,9 +27,10 @@
 #include "dom/dom_misc.h"
 #include "misc/khtmllayout.h"
 #include "misc/shared.h"
+#include "misc/main_thread_malloc.h"
 
-#define QT_ALLOC_QCHAR_VEC( N ) (QChar*) new char[ sizeof(QChar)*( N ) ]
-#define QT_DELETE_QCHAR_VEC( P ) delete[] ((char*)( P ))
+#define QT_ALLOC_QCHAR_VEC( N ) static_cast<QChar*>(main_thread_malloc( sizeof(QChar)*( N ) ))
+#define QT_DELETE_QCHAR_VEC( P ) main_thread_free ((void*)( P ))
 
 namespace DOM {
 
@@ -48,6 +49,8 @@ public:
     DOMStringImpl(const QChar &ch);
     ~DOMStringImpl();
     
+    MAIN_THREAD_ALLOCATED;
+
     unsigned hash() const { if (_hash == 0) _hash = computeHash(s, l); return _hash; }
     static unsigned computeHash(const QChar *, int length);
     static unsigned computeHash(const char *);
