@@ -143,7 +143,7 @@ static int bufferWrite(void* context, const char* buffer, int len)
 
 void XSLTProcessorImpl::addToResult(const char* buffer, int len)
 {
-    m_resultOutput += QString(buffer, len);
+    m_resultOutput += QString::fromUtf8(buffer, len);
 }
 
 DocumentImpl *XSLTProcessorImpl::documentFromXMLDocPtr(xmlDocPtr resultDoc, xsltStylesheetPtr sheet)
@@ -177,7 +177,6 @@ DocumentImpl *XSLTProcessorImpl::documentFromXMLDocPtr(xmlDocPtr resultDoc, xslt
         result->attach();
         result->setURL(m_sourceDocument->URL());
         result->setBaseURL(m_sourceDocument->baseURL());
-        result->setDecoder(m_sourceDocument->decoder()); // FIXME: Should just be UTF-16.
         result->docLoader()->setShowAnimations(m_sourceDocument->docLoader()->showAnimations());
         result->setTransformSourceDocument(m_sourceDocument);
 
