@@ -88,4 +88,10 @@ private:
 #endif
 };
 
+class KWQUIEventTime {
+public:
+    bool uiEventPending() const;
+};
+
+
 #endif
