@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (C) 2004 Apple Computer, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -746,6 +746,84 @@ static int nonMouseButtonsForEvent(NSEvent *event)
     return buttons;
 }
 
+static QPoint positionForEvent(NSEvent *event)
+{
+    switch ([event type]) {
+        case NSLeftMouseDown:
+        case NSLeftMouseUp:
+        case NSLeftMouseDragged:
+        case NSRightMouseDown:
+        case NSRightMouseUp:
+        case NSRightMouseDragged:
+        case NSOtherMouseDown:
+        case NSOtherMouseUp:
+        case NSOtherMouseDragged:
+        case NSMouseMoved:
+        case NSScrollWheel:
+            return QPoint([event locationInWindow]);
+        default:
+            return QPoint();
+    }
+}
+
+static QPoint globalPositionForEvent(NSEvent *event)
+{
+    switch ([event type]) {
+        case NSLeftMouseDown:
+        case NSLeftMouseUp:
+        case NSLeftMouseDragged:
+        case NSRightMouseDown:
+        case NSRightMouseUp:
+        case NSRightMouseDragged:
+        case NSOtherMouseDown:
+        case NSOtherMouseUp:
+        case NSOtherMouseDragged:
+        case NSMouseMoved:
+        case NSScrollWheel:
+            return QPoint([[event window] convertBaseToScreen:[event locationInWindow]]);
+        default:
+            return QPoint();
+    }
+}
+
+static int clickCountForEvent(NSEvent *event)
+{
+    switch ([event type]) {
+        case NSLeftMouseDown:
+        case NSLeftMouseUp:
+        case NSLeftMouseDragged:
+        case NSRightMouseDown:
+        case NSRightMouseUp:
+        case NSRightMouseDragged:
+        case NSOtherMouseDown:
+        case NSOtherMouseUp:
+        case NSOtherMouseDragged:
+            return [event clickCount];
+        default:
+            return 0;
+    }
+}
+
+static Qt::Orientation orientationForEvent(NSEvent *event)
+{
+    switch ([event type]) {
+        case NSScrollWheel:
+            return [event deltaX] != 0 ? Qt::Horizontal : Qt::Vertical;
+        default:
+            return Qt::Vertical;
+    }
+}
+
+static int deltaForEvent(NSEvent *event)
+{
+    switch ([event type]) {
+        case NSScrollWheel:
+            return lrint((orientationForEvent(event) == Qt::Horizontal ? [event deltaX] : [event deltaY]) * 120);
+        default:
+            return 0;
+    }
+}
+
 // ======== 
 
 QEvent::~QEvent()
@@ -755,44 +833,52 @@ QEvent::~QEvent()
 // ======== 
 
 QMouseEvent::QMouseEvent(Type type, const QPoint &position, int button, int state)
-    : QEvent(type), _position(position)
+    : QEvent(type), _position(position), _button(button), _state(state), _clickCount(1)
 {
-    _button = button;
-    if (type == MouseMove) {
-        _clickCount = 0;
-        _state = state | button;
-        _stateAfter = state | button;
-    } else {
-        _clickCount = 1;
-        if (type == MouseButtonRelease) {
-            _state = state | button;
-            _stateAfter = state & ~button;
-        } else {
-            _state = state & ~button;
-            _stateAfter = state | button;
-        }
-    }
+    fixState();
 }
 
 QMouseEvent::QMouseEvent(Type type, NSEvent *event)
-    : QEvent(type), _position([event locationInWindow])
+    : QEvent(type)
+    , _position(positionForEvent(event))
+    , _button(mouseButtonForEvent(event))
+    , _state(nonMouseButtonsForEvent(event))
+    , _clickCount(clickCountForEvent(event))
 {
-    int button = mouseButtonForEvent(event);
-    int state = nonMouseButtonsForEvent(event);
-    _button = button;
-    if (type == MouseMove) {
-        _clickCount = 0;
-        _state = state | button;
-        _stateAfter = state | button;
-    } else {
-        _clickCount = [event clickCount];
-        if (type == MouseButtonRelease) {
+    fixState();
+}
+
+QMouseEvent::QMouseEvent(Type type)
+    : QEvent(type), _button(0), _state(0), _clickCount(0)
+{
+    NSEvent *event = [NSApp currentEvent];
+    if (event) {
+        _position = positionForEvent(event);
+        _button = mouseButtonForEvent(event);
+        _state = nonMouseButtonsForEvent(event);
+        _clickCount = clickCountForEvent(event);
+    }
+    fixState();
+}
+
+void QMouseEvent::fixState()
+{
+    int button = _button;
+    int state = _state;
+    switch (type()) {
+        case MouseMove:
+            _clickCount = 0;
+            _state = state | button;
+            _stateAfter = state | button;
+            break;
+        case MouseButtonRelease:
             _state = state | button;
             _stateAfter = state & ~button;
-        } else {
+            break;
+        default:
             _state = state & ~button;
             _stateAfter = state | button;
-        }
+            break;
     }
 }
 
@@ -816,3 +902,16 @@ QKeyEvent::QKeyEvent(NSEvent *event, bool forceAutoRepeat)
         _unmodifiedText = "\x8";
     }
 }
+
+// ======== 
+
+QWheelEvent::QWheelEvent(NSEvent *event)
+    : QEvent(Wheel)
+    , _position(positionForEvent(event))
+    , _globalPosition(globalPositionForEvent(event))
+    , _delta(deltaForEvent(event))
+    , _state(nonMouseButtonsForEvent(event))
+    , _orientation(orientationForEvent(event))
+    , _isAccepted(false)
+{
+}
