@@ -28,9 +28,12 @@
 #import "KWQButton.h"
 #import "KWQExceptions.h"
 #import "KWQKHTMLPart.h"
-#import "KWQNSViewExtras.h"
 #import "KWQView.h"
 #import "WebCoreBridge.h"
+#import "render_form.h"
+
+using khtml::RenderWidget;
+using khtml::RenderLayer;
 
 @interface KWQSlider : NSSlider <KWQWidgetHolder>
 {
@@ -99,12 +102,16 @@
     BOOL become = [super becomeFirstResponder];
     if (become && slider) {
         if (!KWQKHTMLPart::currentEventIsMouseDownInWidget(slider)) {
-            [self _KWQ_scrollFrameToVisible];
+            RenderWidget *widget = const_cast<RenderWidget *> (static_cast<const RenderWidget *>(slider->eventFilterObject()));
+            RenderLayer *layer = widget->enclosingLayer();
+            if (layer)
+                layer->scrollRectToVisible(widget->absoluteBoundingBoxRect());
         }
 
         if (slider) {
             QFocusEvent event(QEvent::FocusIn);
-            const_cast<QObject *>(slider->eventFilterObject())->eventFilter(slider, &event);
+            if (slider->eventFilterObject())
+                const_cast<QObject *>(slider->eventFilterObject())->eventFilter(slider, &event);
         }
     }
     return become;
@@ -115,7 +122,8 @@
     BOOL resign = [super resignFirstResponder];
     if (resign && slider) {
         QFocusEvent event(QEvent::FocusOut);
-        const_cast<QObject *>(slider->eventFilterObject())->eventFilter(slider, &event);
+        if (slider->eventFilterObject())
+            const_cast<QObject *>(slider->eventFilterObject())->eventFilter(slider, &event);
     }
     return resign;
 }
